data "aws_iam_policy_document" "assume_role" {
  statement {
    effect = "Allow"

    actions = ["sts:AssumeRole"]

    principals {
      type        = "Service"
      identifiers = ["lambda.amazonaws.com"]
    }
  }
}

data "aws_iam_policy_document" "lambda_basic" {
  statement {
    sid = "AllowWriteToCloudwatchLogs"

    effect = "Allow"

    actions = [
      "logs:CreateLogGroup",
      "logs:CreateLogStream",
      "logs:PutLogEvents",
    ]

    resources = ["arn:aws:logs:*:*:*"]
  }

  statement {
    effect = "Allow"

    actions = [
      "sns:Subscribe",
      "sns:ListSubscriptionsByTopic",
      "sns:Receive",
    ]

    resources = ["${aws_sns_topic.slack_notification.arn}"]
  }

  statement {
    effect = "Allow"

    actions = [
    "ssm:GetParameter",
    "ssm:GetParameters",
    "ssm:GetParametersByPath",
    ]

    resources = [
      "arn:aws:ssm:us-west-2:${data.aws_caller_identity.current.account_id}:parameter${var.slack_webhook_url_ssm_path}"
    ]
  }
}

resource "aws_iam_role" "slack_lambda" {
  name_prefix        = "slack_lambda"
  assume_role_policy = "${data.aws_iam_policy_document.assume_role.json}"
}

resource "aws_iam_role_policy" "slack_lambda" {
  name_prefix = "lambda-policy-"
  role        = "${aws_iam_role.slack_lambda.id}"

  policy = "${data.aws_iam_policy_document.lambda_basic.json}"
}